package ldaputil

import (
	"context"
	"time"

	"github.com/cenkalti/backoff"
	"gopkg.in/ldap.v2"

	"git.autistici.org/ai3/go-common/clientutil"
)

// Interface matched by net.Error.
type hasTemporary interface {
	Temporary() bool
}

// Treat network errors as temporary. Other errors are permanent by
// default.
func isTemporaryLDAPError(err error) bool {
	switch v := err.(type) {
	case *ldap.Error:
		switch v.ResultCode {
		case ldap.ErrorNetwork:
			return true
		default:
			return false
		}
	case hasTemporary:
		return v.Temporary()
	default:
		return false
	}
}

// Search performs the given search request. It will retry the request
// on temporary errors.
func (p *ConnectionPool) Search(ctx context.Context, searchRequest *ldap.SearchRequest) (*ldap.SearchResult, error) {
	var result *ldap.SearchResult
	err := clientutil.Retry(func() error {
		conn, err := p.Get(ctx)
		if err != nil {
			// Here conn is nil, so we don't need to Release it.
			if isTemporaryLDAPError(err) {
				return clientutil.TempError(err)
			}
			return err
		}

		if deadline, ok := ctx.Deadline(); ok {
			conn.SetTimeout(time.Until(deadline))
		}

		result, err = conn.Search(searchRequest)
		if err != nil && isTemporaryLDAPError(err) {
			p.Release(conn, err)
			return clientutil.TempError(err)
		}
		p.Release(conn, err)
		return err
	}, backoff.WithContext(clientutil.NewExponentialBackOff(), ctx))
	return result, err
}
